/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       keyfob_main.c
 *    TARGET:     Si4010 RevC
 *    TOOLCHAIN:  Keil
 *    DATE:       Dec 05, 2012,
 *    RELEASE:    1.0 (Tamas Nagy), ROM version 02.00
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the Keil toolchain
 *      4010-kfob-dir project.
 *
 *      BL51 linker directives for building this application:
 *
 *      BL51: PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0x08FF)
 *            DATA (0x40)
 *            IDATA (0x70)
 *            XDATA (0x0900-0x107F)
 *            STACK (?STACK(0x90))
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include "stdlib.h"
#include "si4010.h"
#include "si4010_api_rom.h"

// Demo header
#include "keyfob.h"

void  main (void)
{

/*------------------------------------------------------------------------------
 *    SETUP PHASE
 *------------------------------------------------------------------------------
 */
        //Set DMD interrupt to high priority,
        // any other interrupts have to stay low priority
        PDMD=1;
        // Disable the Matrix and Roff modes on GPIO[3:1] 
        PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
        PORT_CTRL |=  M_PORT_STROBE;
        PORT_CTRL &= (~M_PORT_STROBE);
        // Turn LED control off 
        GPIO_LED = 0;
        vSys_Setup( 10 );
        vSys_SetMasterTime(0);
        // Setup the bandgap 
        vSys_BandGapLdo( 1 );

        if ((PROT0_CTRL & M_NVM_BLOWN) > 1) //if part is burned to user or run mode.
        {
                // Check the first power up bit meaning keyfob is powered on by battery insertion
                if ( 0 != (SYSGEN & M_POWER_1ST_TIME) )
                {
                        vSys_FirstPowerUp(); // Function will shutdown.
                }
        }
        // Set LED intensity. Valid values are 0 (off), 1, 2 and 3 
        vSys_LedIntensity( 3 );
        lLEDOnTime=20;
        //Get production ID (4byte factory burned unique serial number)
        lPartID = lSys_GetProdId();
        lShiftedPartID = (lPartID << 2);
        // Initialize isr call counter 
        bIsr_DebounceCount = 0;

        // BSR parameters initialization 
        rBsrSetup.bButtonMask = bButtonMask_c; 
#ifdef CRYSTAL
        rBsrSetup.bButtonMask &= 0xFE;// clear bit0; GPIO0 will be used by crystal
#endif
        rBsrSetup.pbPtsReserveHead = abBsrPtsPlaceHolder;
        rBsrSetup.bPtsSize = 3;
        rBsrSetup.bPushQualThresh = 3;
        // Setup the BSR 
        vBsr_Setup( &rBsrSetup );

        // Setup the RTC to tick every 5ms and clear it. Keep it disabled. 
        RTC_CTRL = (0x07 << B_RTC_DIV) | M_RTC_CLR;
        // Enable the RTC 
        RTC_CTRL |= M_RTC_ENA;
        // Enable the RTC interrupt and global interrupt enable 
        ERTC = 1;
        EA = 1;

        pbFrameHead = abFrameBuffer;
//------------------------------------------------------------
// Setup RF parameters.
#ifdef PINSELECT
//set SELOUT to zero
SELOUT = 0;
SELOUT = 0;
//FREQSEL pins are open and pulled up internally (==1), or connected to SELOUT or to GND (==0)
//Connection to GND is not recommended because of the excessive current through the internal pullup resistor in shutdown state.
if ((FREQSEL0==1) & (FREQSEL1==1))
{ //433MHz
        fDesiredFreqOOK 	 = f_433_RkeFreqOOK_c;
	fDesiredFreqFSK 	 = f_433_RkeFreqFSK_c;
	bFskDev 		 = b_433_RkeFskDev_c;
	rPaSetup.bLevel      = b_433_PaLevel_c;
  	rPaSetup.wNominalCap = b_433_PaNominalCap_c;
  	rPaSetup.bMaxDrv     = b_433_PaMaxDrv_c;
}
if ((FREQSEL0==0) & (FREQSEL1==1))
{ //868MHz
	fDesiredFreqOOK 	 = f_868_RkeFreqOOK_c;
	fDesiredFreqFSK 	 = f_868_RkeFreqFSK_c;
	bFskDev 		 = b_868_RkeFskDev_c;
	rPaSetup.bLevel      = b_868_PaLevel_c;
  	rPaSetup.wNominalCap = b_868_PaNominalCap_c;
  	rPaSetup.bMaxDrv     = b_868_PaMaxDrv_c;
}
if ((FREQSEL0==1) & (FREQSEL1==0))
{ //315MHz
	fDesiredFreqOOK 	 = f_315_RkeFreqOOK_c;
	fDesiredFreqFSK 	 = f_315_RkeFreqFSK_c;
	bFskDev 		 = b_315_RkeFskDev_c;
	rPaSetup.bLevel      = b_315_PaLevel_c;
  	rPaSetup.wNominalCap = b_315_PaNominalCap_c;
  	rPaSetup.bMaxDrv     = b_315_PaMaxDrv_c;
}
if ((FREQSEL0==0) & (FREQSEL1==0))
{ //915MHz
	fDesiredFreqOOK 	 = f_915_RkeFreqOOK_c;
	fDesiredFreqFSK 	 = f_915_RkeFreqFSK_c;
	bFskDev 		 = b_915_RkeFskDev_c;
	rPaSetup.bLevel      = b_915_PaLevel_c;
  	rPaSetup.wNominalCap = b_915_PaNominalCap_c;
  	rPaSetup.bMaxDrv     = b_915_PaMaxDrv_c;
}
SELOUT = 1;
#else		//if PINSELECT is not used, the following settings are used
        fDesiredFreqOOK 	 = f_433_RkeFreqOOK_c;
        fDesiredFreqFSK 	 = f_433_RkeFreqFSK_c;
        bFskDev 		 = b_433_RkeFskDev_c;
        // Setup the PA.
        rPaSetup.bLevel      = b_433_PaLevel_c;
        rPaSetup.wNominalCap = b_433_PaNominalCap_c;
        rPaSetup.bMaxDrv     = b_433_PaMaxDrv_c;
#endif

        rPaSetup.fAlpha      = 0.0;
        rPaSetup.fBeta       = 0.0;
        vPa_Setup( &rPaSetup );


// Setup frequency casting .. needed to be called once per boot 
vFCast_Setup();

#ifdef CRYSTAL
//XO Setup
rXoSetup.fXoFreq	= 10000000.0;   // Frequency of the external crystal [Hz]
rXoSetup.bLowCap	= 1;			// Capacitance setup of crystal 0 = above 14pf, 1 = below 14pf
vFCast_XoSetup( &rXoSetup );
#endif

// Measure the battery voltage in mV, only once per boot to save power
// Make loaded measurement .. bBatteryWait_c * 17ms wait after loading
iBatteryMv = iMVdd_Measure( bBatteryWait_c );
  
// Setup the DMD temperature sensor for temperature mode 
vDmdTs_RunForTemp( 3 ); // Skip first 3 samples 
// Wait until there is a valid DMD temp sensor sample 
while ( 0 == bDmdTs_GetSamplesTaken() )
{
        //wait
}

 /*------------------------------------------------------------------------------
  *    TRANSMISSION LOOP PHASE
  *------------------------------------------------------------------------------
  */

// Application loop, including push button state analyzation and transmission. 
while(1)
{
        // Buttons analyzation 
        vButtonCheck();
        if (bButtonState)
        {
                // Set frame size and frame pointer 
                vPacketAssemble();
	        if (bValidBtn)
	        {
                        bRepeatCount = bRepeatCount_c;
                        // Start to transmit
                        vRepeatTxLoop();
                }
        }
        else if( (lSys_GetMasterTime() >> 5) > bMaxWaitForPush_c )
        {
	        if ((PROT0_CTRL & M_NVM_BLOWN) > 1) //if part is burned to user or run mode.
  		{
                        //Disable all interrupts
                        EA = 0;
#ifdef CRYSTAL
                        // Disable XO
			bXO_CTRL = 0 ; 		
                        // Wait 20us for XO to stop
                        vSys_16BitDecLoop( 20 );
#endif
                        // Shutdown
                        vSys_Shutdown();      	
		}
        }
}
}


void vRepeatTxLoop (void)
{ 
                // get current timestamp  
                lTimestamp = lSys_GetMasterTime();
                //if part is burned to user or run mode
	        if ((PROT0_CTRL & M_NVM_BLOWN) > 1) 
  	        {
    	                //turn LED on
	 	        GPIO_LED = 1; 
                }	       
                while ( (lSys_GetMasterTime() - lTimestamp) < lLEDOnTime )
                {
                        //wait for LED ON time to expire
                }
	        GPIO_LED = 0;   //turn LED off

        vFCast_Tune( fDesiredFreq );
        if (rOdsSetup.bModulationType == bModFSK_c)
        {
                vFCast_FskAdj( bFskDev ); 
        }
        // Wait until there is a temperature sample available
        while ( 0 == bDmdTs_GetSamplesTaken() )
        {
                //wait
        }
        //  Tune the PA with the temperature as an argument 
        vPa_Tune( iDmdTs_GetLatestTemp());
        vStl_PreLoop();

        do
        {
                // get current timestamp  
                lTimestamp = lSys_GetMasterTime();
	        //Transmit packet
                vStl_SingleTxLoop(pbFrameHead,bFrameSize);
	        // Wait repeat interval. 
                while ( (lSys_GetMasterTime() - lTimestamp) < wRepeatInterval_c );

        }while(--bRepeatCount);

        vStl_PostLoop();
                // get current timestamp  
                lTimestamp = lSys_GetMasterTime();
                //if part is burned to user or run mode
	        if ((PROT0_CTRL & M_NVM_BLOWN) > 1) 
  	        {
    	                //turn LED on
	 	        GPIO_LED = 1; 
                }	       
                while ( (lSys_GetMasterTime() - lTimestamp) < lLEDOnTime )
                {
                        //wait for LED ON time to expire
                }
	        GPIO_LED = 0;   //turn LED off

         // Clear time value for next button push detecting. 
        vSys_SetMasterTime(0);
        return;
} 

//------------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void isr_rtc (void) interrupt INTERRUPT_RTC using 1

{ 

  // Update the master time by 5 every time this isr is run.
  // clear the RTC_INT 
  RTC_CTRL &= ~M_RTC_INT;
  vSys_IncMasterTime(5); 
  bIsr_DebounceCount ++;
  if ((bIsr_DebounceCount % bDebounceInterval_c) == 0)
  {
    vBsr_Service();
  }
  return;
}
 
/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Update bAp_ButtonState which indicates what to be transmitted.
 *      Check the elements on PTS (push tracking strcture) to see if any GPIO
 *      has been pressed or released.
 *      If any new pressed button has detected, the corresponding flag will be set and
 *      the associated information will be transmitted in
 *      application loop procedure.
 *
 *
 *------------------------------------------------------------------------------ 
 */

void vButtonCheck (void)
{ 
  // Disable RTC interrupt, or button state might be updated. 
  ERTC = 0;
  bButtonState = 0;  //comment this line out if autorepeat needed
  if (bBsr_GetPtsItemCnt())
     //Some buttons were pressed or released
  {
    
	bButtonState = wBsr_Pop() & 0xFF;
	if (bPrevBtn)
	{
		bPrevBtn = bButtonState;
		bButtonState = 0;
	}
	else
	{
		bPrevBtn = bButtonState;
	}
  }
  // Enable RTC interrupt 
  ERTC = 1;
  return;
}

/*
 *------------------------------------------------------------------------------ 
 */
void vPacketAssemble (void)
{ 
        BYTE i, j;

        //according to button pushed, insert data bits in packet
        switch (bButtonState)
        {
                case bRightButton_c: 
                case bLeftButton_c:        
                        vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );
                        rOdsSetup.bModulationType = bModFSK_c;
                        fDesiredFreq = fDesiredFreqFSK;
                        bPreamble = bPreambleNrz_c;
                        break;
                case bMiddleButton_c:       
                        vStl_EncodeSetup( bEnc_Manchester_c, NULL );
                        rOdsSetup.bModulationType = bModOOK_c;
                        fDesiredFreq = fDesiredFreqOOK;
                        bPreamble = bPreambleManch_c;
                        break;
                case bRearButton_c:        //HT6P20 OOK 
            	        pbRData[3] = ((BYTE *) &lShiftedPartID)[3];
                        pbRData[3] |= 0x01;		//Insert button information
                        break;
                case bFrontButton_c:        //HT6P20 OOK 
            	        pbRData[3] = ((BYTE *) &lShiftedPartID)[3];
                        pbRData[3] |= 0x02;		//Insert button information
                        break;
        }
        switch (bButtonState)
        {
                case bRightButton_c:
                case bMiddleButton_c:
                case bLeftButton_c:
                        //clear button bits in bStatus
                        bStatus &= ~M_ButtonBits_c;
                        //copy button bits from bButtonState to bStatus
                        bStatus |= bButtonState & M_ButtonBits_c;
                        //Normal packet, FSK or OOK
	                bFrameSize = bFrameSize_c;

                        abFrameBuffer[0] = bPreamble;	
                        abFrameBuffer[1] = bPreamble;	
                        abFrameBuffer[2] = bPreamble;	
                        abFrameBuffer[3] = bPreamble;	
                        abFrameBuffer[4] = bPreamble;	
                        abFrameBuffer[5] = bSync1_c;		
                        abFrameBuffer[6] = bSync2_c;		
	                abFrameBuffer[7] = ((BYTE *)&lPartID)[0];
	                abFrameBuffer[8] = ((BYTE *)&lPartID)[1];
	                abFrameBuffer[9] = ((BYTE *)&lPartID)[2];
	                abFrameBuffer[10] = ((BYTE *)&lPartID)[3];
	                abFrameBuffer[11] = bStatus;
	                abFrameBuffer[12] = ((BYTE *)&iBatteryMv)[0];
	                abFrameBuffer[13] = ((BYTE *)&iBatteryMv)[1];
	                abFrameBuffer[14] = 0;	//CRC
	                abFrameBuffer[15] = 0;	//CRC

	                vCalculateCrc();
	                vConvertPacket(rOdsSetup.bModulationType);

                        // 24MHz / (bClkDiv+1) / 9.6kbps = 417 
                        rOdsSetup.wBitRate      = 417;
                        rOdsSetup.bClkDiv       = 5;
                        rOdsSetup.bEdgeRate     = 0;
                        rOdsSetup.bGroupWidth   = 7;
                        rOdsSetup.bLcWarmInt    = 0;
                        rOdsSetup.bDivWarmInt   = 5;
                        rOdsSetup.bPaWarmInt    = 4;
                        vOds_Setup( &rOdsSetup );
	                bValidBtn = 1;
                        break;
                case bRearButton_c:        
                case bFrontButton_c:        
                        //HT6P20 OOK
                        vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );
                        // Setup the ODS for HT6P20 (OOK)
                        rOdsSetup.bModulationType = bModOOK_c; // Use OOK 
                        fDesiredFreq = fDesiredFreqOOK;

	                bFrameSize = bHFrameSize_c;
                        // lPartID is in big endian fashion
    	                pbRData[2] = ((BYTE *) &lShiftedPartID)[2];
    	                pbRData[1] = ((BYTE *) &lShiftedPartID)[1];
    	                pbRData[0] = ((BYTE *) &lShiftedPartID)[0]; //won't be transmitted
                        // Fill frame buffer with the holtek packet
                        abFrameBuffer[0] = 0x00;		//pilot period 
                        abFrameBuffer[1] = 0x00;		//pilot period
                        abFrameBuffer[2] = 0x00;		//pilot period 
                        abFrameBuffer[3] = 0x20;		//pilot period + start bit
                        //convert each address data bit to three bits 
                        //(also MSB first to LSB first conversion is included)
	                for (i=15,j=3;i>3;i-=4,j--)
	                {
	                abFrameBuffer[i]=ab2to6bitTable[pbRData[j] & 0x03];
	                abFrameBuffer[i-1]=ab2to6bitTable[(pbRData[j] >> 2) & 0x03];
	                abFrameBuffer[i-2]=ab2to6bitTable[(pbRData[j] >> 4) & 0x03];
	                abFrameBuffer[i-3]=ab2to6bitTable[(pbRData[j] >> 6) & 0x03];
	                }
                        abFrameBuffer[16] = 0x26;		//anti-code period
                        abFrameBuffer[17] = 0x26;		//anti-code period

                        rOdsSetup.bClkDiv         = 5;
                        rOdsSetup.bEdgeRate       = 0;
                        // Set group width to 5, which means 6 bits/encoded byte to be transmitted.
                        rOdsSetup.bGroupWidth     = 5;
                        // 24MHz / (bClkDiv+1) * 467us = 1868 
                        rOdsSetup.wBitRate        = 1868;
                        // Configure the warm up intervals LC: 8, DIV: 5, PA: 4 
                        rOdsSetup.bLcWarmInt  = 8;
                        rOdsSetup.bDivWarmInt = 5;
                        rOdsSetup.bPaWarmInt  = 4;
                        // ODS setup 
                        vOds_Setup( &rOdsSetup );
                        bValidBtn = 1;
                        break;
                default:
                        pbRData[0] = pbRData[0] | 0xc0;
	                bValidBtn = 0;
                        break;

        }
        return;
}
//--------------------------------------------------------------
//Calculate CRC and write in the frame buffer
//Bit pattern used (1)1000 0000 0000 0101, X16+X15+X2+1
void vCalculateCrc(void)
{
        BYTE i,j;
        WORD wCrc;
        wCrc = 0xffff;
        for(j = bPayloadStartIndex_c;j<bPayloadStartIndex_c + bPayloadSize_c;j++)
        {
                wCrc = wCrc ^ ((WORD)abFrameBuffer[j]<<8);

                for (i = 8; i != 0; i--)
                {
                        if (wCrc & 0x8000)
                        {
	                        wCrc = (wCrc << 1) ^ 0x8005;
                        }
                        else
	                {
                                wCrc <<= 1;
                        }
                }
        }
  //-----------------------------------------------------------------
  //Write CRC in frame
  abFrameBuffer[bFrameSize_c - 2] = ((BYTE*)&wCrc)[0];
  abFrameBuffer[bFrameSize_c - 1] = ((BYTE*)&wCrc)[1];
  return;
}

  //-------------------------------------------------------------------
  //MSB first to LSB first conversion, and inversion if FSK used
void vConvertPacket (BYTE bModType)
{
  BYTE i,low,high;

  if (bModType)
  {
    bModType = 0xff;
  }
  for (i=0;i<(sizeof abFrameBuffer);i++)
  {
	low = abConvTable[(abFrameBuffer[i] & 0xf0) >> 4] & 0x0f;
	high = abConvTable[abFrameBuffer[i] & 0x0f] & 0xf0;
	abFrameBuffer[i] = (high | low) ^ bModType;
  }
  return;
}
