/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                               Copyright 2010
 *                         CONFIDENTIAL & PROPRIETARY
 *------------------------------------------------------------------------------
 *
 *    FILE:       OD_main.h
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    RELEASE:    1.1 ; ROM version 02.00
 *    AUTHOR:     Miklos Lukacs
 *    DATE:       Jun 07, 2010
 *    EDITOR:     Viktor Huszar
 *    DATE:       Aug 12, 2011 (Add: BMSR)
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      Header file for the keyfob_demo module.
 *
 *------------------------------------------------------------------------------*/
#ifndef _KEYFOB_DEMO_H
#define _KEYFOB_DEMO_H

/*------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include "si4010_types.h"

/*------------------------------------------------------------------------------
 *
 *    Definitions:
 */

#undef BOARD_TEST
#undef RUN_FIFO_TEST

#define PART_FACTORY 	0x01
#define PART_USER 		0x03
#define PART_RUN 		  0x07

#define bRke_EncodeNoneNrz_c       0   /* No encoding */
#define bRke_EncodeManchester_c    1   /* Manchester encoding */
#define bRke_Encode4b5b_c          2   /* 4b-5b encoding */

#define bRke_OokMod_c        (0x00)
#define bRke_FskMod_c        (0x01)

//Row masks
#define bMatrixRow1_c             0x40
#define bMatrixRow2_c             0x80
#define bMatrixRow3_c             0xC0

//Column masks
#define bMatrixCol1_c             0x01
#define bMatrixCol2_c             0x02
#define bMatrixCol3_c             0x04
#define bMatrixCol4_c             0x08
#define bMatrixCol5_c             0x10
#define bMatrixCol6_c             0x20

// definition of the RF packet length
#define bFrameSize_c            8

// definition of the PTS length
#define bPtsSize_c            	(10U)

//Control byte of receiver's diode
#define bDiode1_NC_c			0x00  //diode no change
#define bDiode2_NC_c			0x00
#define bDiode3_NC_c			0x00
#define bDiode4_NC_c			0x00
#define bDiode1_OFF_c			0x80  //diode switch OFF
#define bDiode2_OFF_c			0x20
#define bDiode3_OFF_c			0x08
#define bDiode4_OFF_c			0x02
#define bDiode1_ON_c			0x40  //diode switch ON
#define bDiode2_ON_c			0x10
#define bDiode3_ON_c			0x04
#define bDiode4_ON_c			0x01
#define bDiode1_FL_c			0xC0  //diode mono-stable mode, 100ms
#define bDiode2_FL_c			0x30
#define bDiode3_FL_c			0x0C
#define bDiode4_FL_c			0x03

//Control byte of receiver's diode code in packets
#define bDiode_0001_FL_c		(bDiode1_FL_c 	| bDiode2_OFF_c	| bDiode3_OFF_c | bDiode4_OFF_c)
#define bDiode_0010_FL_c		(bDiode1_OFF_c 	| bDiode2_FL_c	| bDiode3_OFF_c | bDiode4_OFF_c)
#define bDiode_0011_FL_c		(bDiode1_FL_c 	| bDiode2_FL_c 	| bDiode3_OFF_c | bDiode4_OFF_c)
#define bDiode_0100_FL_c		(bDiode1_OFF_c 	| bDiode2_OFF_c | bDiode3_FL_c 	| bDiode4_OFF_c)
#define bDiode_0101_FL_c		(bDiode1_FL_c 	| bDiode2_OFF_c | bDiode3_FL_c 	| bDiode4_OFF_c)
#define bDiode_0110_FL_c		(bDiode1_OFF_c 	| bDiode2_FL_c	| bDiode3_FL_c 	| bDiode4_OFF_c)
#define bDiode_0111_FL_c		(bDiode1_FL_c 	| bDiode2_FL_c 	| bDiode3_FL_c 	| bDiode4_OFF_c)
#define bDiode_1000_FL_c		(bDiode1_OFF_c 	| bDiode2_OFF_c | bDiode3_OFF_c | bDiode4_FL_c)
#define bDiode_1001_FL_c		(bDiode1_FL_c 	| bDiode2_OFF_c | bDiode3_OFF_c | bDiode4_FL_c)
#define bDiode_1010_FL_c		(bDiode1_OFF_c 	| bDiode2_FL_c 	| bDiode3_OFF_c | bDiode4_FL_c)
#define bDiode_1011_FL_c		(bDiode1_FL_c 	| bDiode2_FL_c 	| bDiode3_OFF_c | bDiode4_FL_c)
#define bDiode_1100_FL_c		(bDiode1_OFF_c 	| bDiode2_OFF_c | bDiode3_FL_c 	| bDiode4_FL_c)
#define bDiode_1101_FL_c		(bDiode1_FL_c 	| bDiode2_OFF_c | bDiode3_FL_c 	| bDiode4_FL_c)
#define bDiode_1110_FL_c		(bDiode1_OFF_c 	| bDiode2_FL_c 	| bDiode3_FL_c 	| bDiode4_FL_c)
#define bDiode_1111_FL_c		(bDiode1_FL_c 	| bDiode2_FL_c 	| bDiode3_FL_c 	| bDiode4_FL_c)

#define bDiode_0010_ON_c		(bDiode1_OFF_c 	| bDiode2_ON_c	| bDiode3_OFF_c | bDiode4_OFF_c)
#define bDiode_0100_ON_c		(bDiode1_OFF_c 	| bDiode2_OFF_c | bDiode3_ON_c 	| bDiode4_OFF_c)
#define bDiode_1000_ON_c		(bDiode1_OFF_c 	| bDiode2_OFF_c	| bDiode3_OFF_c | bDiode4_ON_c)

// RF packet fields
#define lFramePreamble_c        0xAAAAAA
#define wFrameSyncWord_c        0x4BD4
#define lFramePayLoad01_c        ~( ((LWORD)bDiode_0001_FL_c << 16) | ((~bDiode_0001_FL_c) << 8) | (bDiode_0001_FL_c))
#define lFramePayLoad02_c        ~( ((LWORD)bDiode_0010_FL_c << 16) | ((~bDiode_0010_FL_c) << 8) | (bDiode_0010_FL_c))
#define lFramePayLoad03_c        ~( ((LWORD)bDiode_0011_FL_c << 16) | ((~bDiode_0011_FL_c) << 8) | (bDiode_0011_FL_c))
#define lFramePayLoad04_c        ~( ((LWORD)bDiode_0100_FL_c << 16) | ((~bDiode_0100_FL_c) << 8) | (bDiode_0100_FL_c))
#define lFramePayLoad05_c        ~( ((LWORD)bDiode_0101_FL_c << 16) | ((~bDiode_0101_FL_c) << 8) | (bDiode_0101_FL_c))
#define lFramePayLoad06_c        ~( ((LWORD)bDiode_0110_FL_c << 16) | ((~bDiode_0110_FL_c) << 8) | (bDiode_0110_FL_c))
#define lFramePayLoad07_c        ~( ((LWORD)bDiode_0111_FL_c << 16) | ((~bDiode_0111_FL_c) << 8) | (bDiode_0111_FL_c))
#define lFramePayLoad08_c        ~( ((LWORD)bDiode_1000_FL_c << 16) | ((~bDiode_1000_FL_c) << 8) | (bDiode_1000_FL_c))
#define lFramePayLoad09_c        ~( ((LWORD)bDiode_1001_FL_c << 16) | ((~bDiode_1001_FL_c) << 8) | (bDiode_1001_FL_c))
#define lFramePayLoad10_c        ~( ((LWORD)bDiode_1010_FL_c << 16) | ((~bDiode_1010_FL_c) << 8) | (bDiode_1010_FL_c))
#define lFramePayLoad11_c        ~( ((LWORD)bDiode_1011_FL_c << 16) | ((~bDiode_1011_FL_c) << 8) | (bDiode_1011_FL_c))
#define lFramePayLoad12_c        ~( ((LWORD)bDiode_1100_FL_c << 16) | ((~bDiode_1100_FL_c) << 8) | (bDiode_1100_FL_c))
#define lFramePayLoad13_c        ~( ((LWORD)bDiode_1101_FL_c << 16) | ((~bDiode_1101_FL_c) << 8) | (bDiode_1101_FL_c))
#define lFramePayLoad14_c        ~( ((LWORD)bDiode_1110_FL_c << 16) | ((~bDiode_1110_FL_c) << 8) | (bDiode_1110_FL_c))
#define lFramePayLoad15_c        ~( ((LWORD)bDiode_1111_FL_c << 16) | ((~bDiode_1111_FL_c) << 8) | (bDiode_1111_FL_c))
#define lFramePayLoad16_c        ~( ((LWORD)bDiode_0010_ON_c << 16) | ((~bDiode_0010_ON_c) << 8) | (bDiode_0010_ON_c))
#define lFramePayLoad17_c        ~( ((LWORD)bDiode_0100_ON_c << 16) | ((~bDiode_0100_ON_c) << 8) | (bDiode_0100_ON_c))
#define lFramePayLoad18_c        ~( ((LWORD)bDiode_1000_ON_c << 16) | ((~bDiode_1000_ON_c) << 8) | (bDiode_1000_ON_c))

// Amount of time (x32ms), the application will wait after boot without
// getting a qualified button push before giving up and shutting the chip down.
// Sets the maximum wait time for push as 1.6s; the variable is unit in 32ms.
#define	bMaxWaitForPush_c		50

// This specifies the number of times that the RTC ISR is called between 
// each call to the button service routine.  The actual time between
// calls is dependent on the setting of RTC_CTRL.RTC_DIV which dictates how
// often the RTC ISR is called.    
#define	bDebounceInterval_c		8

// Sets target frequency to 433.46MHz.
#define fOperatingFrequency  	433460000.0

// Sets FSK deviation 38kHz, here arg 40 is calculated by spreadsheed
// and will be used for FSK adjustment.
#define bFskDeviation 			127

// The time of packet transmission for the demo:
// 8 bytes / 8 bits / 9.6kbps = 6.67 ms
// Here set the packet interval about 15ms.
#define wRepeatInterval 		15

// Repeat packet 8 times for each button packet transmission.
#define bRepeatCount_c  		8



/*------------------------------------------------------------------------------
 *
 *    VARIABLES:
 */

// Packet transmit repeat counter 
 BYTE xdata bRepeatCount;
 
 LWORD xdata lTimestamp;

// storing the button which is under processing
 BYTE xdata bAp_ButtonState;

// application run state: 0 - Stores button entries to PTS; 1 - Send PTS entries to receiver
 BYTE xdata bAp_RunState;

// Pointer to the head of the frame to be sent out.
// the vStl_SingleTxLoop() API function expects a pointer residing in the Data memory
 BYTE xdata *pbRke_FrameHead;

// Structure for setting up the ODS .. must be in XDATA //
  tOds_Setup xdata rOdsSetup;

// Structure for setting up the PA .. must be in XDATA //
  tPa_Setup xdata  rPaSetup;

// Battery measurement .. delete if battery measurement is not used //
  int           iBatteryMv;
  BYTE          bBatteryWait;   // Wait time after loading in 17us increment //

// BMSR control structure //
  tBMsr_Setup      xdata rBMsrSetup;
  tBMsr_PtsElement xdata arBMsrPtsPlaceHolder [bPtsSize_c] = {0};

// setup structure for external crystal oscillator //
  tFCast_XoSetup rFCast_XoSetup;

 BYTE bIsr_DebounceCount;
// BYTE bdata bPrevBtn = 0;
// BYTE bPrevBtn = 0;
 BYTE bdata bPrevBtn = 0x10;

// array for storing the TX packet
// needs to be present in the global variable area because it is used in more 
// than one Overly Module
 BYTE  xdata abButtonFrame[bFrameSize_c];

// storing the payload of the RF packet, and used as a return status of the 
// vPacketAssemble() function
 LWORD xdata lPayLoad;



/*-------------------------------------------------------------------------
 *
 *    FUNCTION PROTOTYPES:										
 */

void vAppInitial(void);

void vButtonCheck(void);

void vPacketAssemble(void);

void vButtonCheck(void);

void vRtl_RepeatTxLoop(void);


#endif /* _KEYFOB_DEMO_H */
