
/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2011
 *------------------------------------------------------------------------------
 *
 *    FILE:       si4010_api_BMsr.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  
 *    RELEASE:    v1.1
 *    AUTHOR:     Viktor Huszar
 *    DATE:       Jun 03, 2013
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the additional API libraries required for 
 *		Button-Matrix applications.
 *
 *      BL51 linker directives for building this application:
 *
 *      BL51: PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0x08FF)
 *            DATA (0x40)
 *            IDATA (0x80)
 *            XDATA (0x0900-0x107F)
 *            STACK (?STACK(0x90))
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"
#include "si4010_api_BMsr.h"


#define GPIO_ROW1 GPIO1
#define GPIO_ROW2 GPIO2
#define GPIO_ROW3 GPIO3


#define bBMsr_ColOfPushedButton bBMsr_MatrixCol

/* Bit address and mask to place button-matrix columns to [5:0] */

//#define vBMsr_ReadMatrixCol bBMsr_MatrixCol = 0;
#define vBMsr_ReadMatrixCol bBMsr_MatrixCol_bit5 = ~GPIO9; \
									          bBMsr_MatrixCol_bit4 = ~GPIO8; \
									          bBMsr_MatrixCol_bit3 = ~GPIO7; \
									          bBMsr_MatrixCol_bit2 = ~GPIO6; \
									          bBMsr_MatrixCol_bit1 = ~GPIO4; \
									          bBMsr_MatrixCol_bit0 = ~GPIO0

#define bBMsr_MatrixRow1Used (bBMsr_MatrixRow1Used_bit)
#define bBMsr_MatrixRow2Used (bBMsr_MatrixRow2Used_bit)
#define bBMsr_MatrixRow3Used (bBMsr_MatrixRow3Used_bit)

#define bBMsr_MatrixRow1_c 0x40
#define bBMsr_MatrixRow2_c 0x80
#define bBMsr_MatrixRow3_c 0xC0

/* Internal variables */
//static BYTE xdata bBMsr_DebounceCntr = 0; /* Counts valid datas to verify button push/release */
//static BYTE xdata bBMsr_ButtonToDebounce = 0; /* Stores the pushed/released button to debounce it */
//static BYTE xdata bBMsr_RowMPX = 0; 	/* Multiplex address of rows in button-matrix */
//static BYTE xdata bBMsr_PtsCntr = 0;	/* Count of values stored in PTS */
//static BYTE xdata bBMsr_PtsFirst = 0; /* Points to firs element of actual PTS FIFO */
//static BYTE xdata bBMsr_FirstButton = 1; /* Points to firs element of actual PTS FIFO */

//static tBMsr_Setup xdata *prBMsr_Setup_Data = NULL; /* Pointer to user's setup structure */
//static tBMsr_PtsElement xdata rBMsr_LastButton;

static BYTE bBMsr_DebounceCntr = 0; /* Counts valid datas to verify button push/release */
static BYTE bBMsr_ButtonToDebounce = 0; /* Stores the pushed/released button to debounce it */
static BYTE bBMsr_PtsCntr = 0;	/* Count of values stored in PTS */
static BYTE bBMsr_PtsFirst = 0; /* Points to firs element of actual PTS FIFO */
static BYTE bBMsr_FirstButton = 1; /* Points to firs element of actual PTS FIFO */
static BYTE bBMsr_RowMPX = 0; 	/* Multiplex address of rows in button-matrix */

static tBMsr_Setup *prBMsr_Setup_Data = NULL; /* Pointer to user's setup structure */
static tBMsr_PtsElement rBMsr_LastButton;

static bit bBMsr_MatrixRow1Used_bit = 0;
static bit bBMsr_MatrixRow2Used_bit = 0;
static bit bBMsr_MatrixRow3Used_bit = 0;

static BYTE bdata bBMsr_MatrixCol = 0;
sbit bBMsr_MatrixCol_bit0 = bBMsr_MatrixCol^0;
sbit bBMsr_MatrixCol_bit1 = bBMsr_MatrixCol^1;
sbit bBMsr_MatrixCol_bit2 = bBMsr_MatrixCol^2;
sbit bBMsr_MatrixCol_bit3 = bBMsr_MatrixCol^3;
sbit bBMsr_MatrixCol_bit4 = bBMsr_MatrixCol^4;
sbit bBMsr_MatrixCol_bit5 = bBMsr_MatrixCol^5;

static BYTE bdata bBMsr_MatrixColMask = 0;


/* Returns WORD: MSB timestamp, LSB button vector of the latest entry in PTS */
WORD    wBMsr_Pop (void)
{
	WORD bBMsr_Pop_ret;	

	if (bBMsr_PtsCntr > 0) //There are items in PTS
	{	
		bBMsr_Pop_ret = 	
			((WORD)prBMsr_Setup_Data->prPtsReserveHead[bBMsr_PtsFirst].bTimestamp << 8 ) +
			(prBMsr_Setup_Data->prPtsReserveHead[bBMsr_PtsFirst].bButtonVector);
		bBMsr_PtsFirst = (BYTE)((++bBMsr_PtsFirst) % prBMsr_Setup_Data->bPtsSize);
		bBMsr_PtsCntr--;
	}
	else
		bBMsr_Pop_ret = 0;	

	return bBMsr_Pop_ret;
}

/* Sets up internal variables and PTS*/
void    vBMsr_Setup(tBMsr_Setup xdata *priSetup)
{
	if (prBMsr_Setup_Data != NULL)
		free(prBMsr_Setup_Data);
		
	prBMsr_Setup_Data = priSetup;

  bBMsr_ButtonToDebounce  = 0;
  bBMsr_MatrixCol         = 0;
  bBMsr_MatrixColMask = prBMsr_Setup_Data->bMatrixColMask;

  bBMsr_MatrixRow1Used_bit = (bit)(0x01 & (prBMsr_Setup_Data->bMatrixRowMask));
  bBMsr_MatrixRow2Used_bit = (bit)(0x02 & (prBMsr_Setup_Data->bMatrixRowMask));
  bBMsr_MatrixRow3Used_bit = (bit)(0x04 & (prBMsr_Setup_Data->bMatrixRowMask));

  GPIO_ROW1 = 1;
  GPIO_ROW2 = 1;
  GPIO_ROW3 = 1;

  //Select the first used row for MUX
	if (bBMsr_MatrixRow1Used) 
  {
    bBMsr_RowMPX = bBMsr_MatrixRow1_c;
		GPIO_ROW1 = 0;
  }
	else if (bBMsr_MatrixRow2Used) 
  {
    bBMsr_RowMPX = bBMsr_MatrixRow2_c;
		GPIO_ROW2 = 0;
  }
	else if (bBMsr_MatrixRow3Used) 
  {
    bBMsr_RowMPX = bBMsr_MatrixRow3_c;
		GPIO_ROW3 = 0;
  }
	else 
  {
    bBMsr_RowMPX = 0;
  }

  //Initialize PTS
	vBMsr_InitPts(prBMsr_Setup_Data->bPtsSize);
	
}

/* Initialize PTS and item pointers*/ 
void    vBMsr_InitPts(BYTE biPtsSize)
{
	BYTE bBMsr_Setup_Cntr = biPtsSize-1;

	while (bBMsr_Setup_Cntr != 0)
	{
		prBMsr_Setup_Data->prPtsReserveHead[bBMsr_Setup_Cntr].bButtonVector = 0;
		prBMsr_Setup_Data->prPtsReserveHead[bBMsr_Setup_Cntr--].bTimestamp  = 0;
	}

	bBMsr_DebounceCntr = 0;
	bBMsr_PtsCntr = 0;
	bBMsr_PtsFirst = 0;

	rBMsr_LastButton.bButtonVector 	= 0;
	rBMsr_LastButton.bTimestamp 	= 0;

}


/* Returns BYTE: Number of unpopped entries in PTS */
BYTE    bBMsr_GetPtsItemCnt(void)
{
	return bBMsr_PtsCntr;
}

/* BMSR Service: makes debounce, stores new items into PTS, 
   changes the row MUX if no button pressed*/ 
void    vBMsr_Service(void)
{
	BYTE bBMsr_Last_temp;
  
  vBMsr_ReadMatrixCol;

	bBMsr_ColOfPushedButton = bBMsr_MatrixCol & bBMsr_MatrixColMask; //Mask columns
	
		if (bBMsr_ColOfPushedButton == bBMsr_ButtonToDebounce) //Pushed button is the same as it was
		{
			if (++bBMsr_DebounceCntr == prBMsr_Setup_Data->bPushQualThresh) //Debounce is just fulfilled
			{
				if (!(bBMsr_FirstButton && (bBMsr_ColOfPushedButton == 0x00))) //Not to store first "release" just after swithc ON
				{
					if (bBMsr_PtsCntr < prBMsr_Setup_Data->bPtsSize) //There are more free place in the PTS
					{
						bBMsr_Last_temp = (BYTE)((bBMsr_PtsFirst + bBMsr_PtsCntr) % prBMsr_Setup_Data->bPtsSize);

						prBMsr_Setup_Data->prPtsReserveHead[bBMsr_Last_temp].bButtonVector 	= 
						rBMsr_LastButton.bButtonVector 										= 
						bBMsr_RowMPX | bBMsr_ColOfPushedButton;

						prBMsr_Setup_Data->prPtsReserveHead[bBMsr_Last_temp].bTimestamp 	= 			
						rBMsr_LastButton.bTimestamp 										= 
						bBMsr_GetTimestamp();
						
						bBMsr_PtsCntr++;
					}
					else //PTS is full, store only last pushed item
					{
            if (bBMsr_ColOfPushedButton == 0x00) //PTS is full, but all button was released 
                                                 //  --> clear PTS and store 'all button release' item with timestamp            
            {
              vBMsr_InitPts(prBMsr_Setup_Data->bPtsSize);

            	bBMsr_PtsCntr = 1;

              bBMsr_FirstButton = 1;

  						prBMsr_Setup_Data->prPtsReserveHead[0].bButtonVector 	= 
  						rBMsr_LastButton.bButtonVector 										= 
  						0x00;

  						prBMsr_Setup_Data->prPtsReserveHead[0].bTimestamp 	= 			
  						rBMsr_LastButton.bTimestamp 										= 
  						bBMsr_GetTimestamp();
            }
            else
            {
						  rBMsr_LastButton.bButtonVector 	= bBMsr_RowMPX | bBMsr_ColOfPushedButton;;
						  rBMsr_LastButton.bTimestamp 	= bBMsr_GetTimestamp();
            }
				    
					}
				}
			}
			else if(bBMsr_DebounceCntr > prBMsr_Setup_Data->bPushQualThresh) //No button is pushed and debounce of release is done
			{
				if (bBMsr_FirstButton) bBMsr_FirstButton = 0;

				if (bBMsr_ColOfPushedButton == 0x00)
				{
					bBMsr_DebounceCntr--;

					//Find out next row to be used
					switch (bBMsr_RowMPX)
					{
						case bBMsr_MatrixRow1_c: 
              if (bBMsr_MatrixRow2Used) 
                bBMsr_RowMPX = bBMsr_MatrixRow2_c;
              else if (bBMsr_MatrixRow3Used) 
                bBMsr_RowMPX = bBMsr_MatrixRow3_c;
							break;
						case bBMsr_MatrixRow2_c: 
              if (bBMsr_MatrixRow3Used) 
                bBMsr_RowMPX = bBMsr_MatrixRow3_c;
              else if (bBMsr_MatrixRow1Used) 
                bBMsr_RowMPX = bBMsr_MatrixRow1_c;
							break;
						case bBMsr_MatrixRow3_c: 
              if (bBMsr_MatrixRow1Used) 
                bBMsr_RowMPX = bBMsr_MatrixRow1_c;
              else if (bBMsr_MatrixRow2Used) 
                bBMsr_RowMPX = bBMsr_MatrixRow2_c;
							break;
						default:
							bBMsr_RowMPX = 0;
							break;
					}

          GPIO_ROW1 = 1;
          GPIO_ROW2 = 1;
          GPIO_ROW3 = 1;

					//Select next row
					switch (bBMsr_RowMPX)
					{
						case bBMsr_MatrixRow1_c: 
							GPIO_ROW1 = 0;
							break;
						case bBMsr_MatrixRow2_c: 
							GPIO_ROW2 = 0;
							break;
						case bBMsr_MatrixRow3_c: 
							GPIO_ROW3 = 0;
							break;
						default:
							break;
					}
				}
			}
		}
		else //Pushed button was changed
		{
			bBMsr_DebounceCntr = 1;
			bBMsr_ButtonToDebounce = bBMsr_ColOfPushedButton;
		}

}


/* Returns WORD: MSB timestamp, LSB button vector of the latest button push */
WORD    wBMsr_GetCurrentButton(void)
{
	return (WORD)( ((WORD)rBMsr_LastButton.bTimestamp << 8) + (rBMsr_LastButton.bButtonVector) );
}


BYTE    bBMsr_GetTimestamp(void)
{
  WORD temp = (WORD) lSys_GetMasterTime();
	return (BYTE)( temp >> 5); // MasterTime / 32
}



