
#ifndef _SI4010_API_BMSR
#define _SI4010_API_BMSR
/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2011
 *------------------------------------------------------------------------------
 *
 *    FILE:       si4010_api_BMsr.h
 *    TARGET:     Si4010
 *    TOOLCHAIN:  
 *    RELEASE:    v1.1
 *    AUTHOR:     Viktor Huszar
 *    DATE:       Jun 03, 2013
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This header file is to be used with the si4010 BMSR API and 
 *      to be included to all the files uses BMSR functions.
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include "si4010.h"

/*
 *------------------------------------------------------------------------------
 *
 *    MACROS:
 */


/*
 *------------------------------------------------------------------------------
 *
 *    TYPE DEFINITIONS:
 */

/* ---- Button-Matrix Service Routine */

/* An element of the internal Push Tracking Structure (PTS) */
typedef struct {
  BYTE  bButtonVector;  /* Vector of bits representing each button, [7:6] are for rows and [5:0] are for columns */
  BYTE  bTimestamp;     /* Timestamp associated with button vector */
} tBMsr_PtsElement;

/* Structure for BMSR setup. Space for Push Tracking Structure (PTS)
 * must be reserved by a designer in XDATA prior to initializing BMSR. */
typedef struct {
  BYTE          bMatrixRowMask;     /* Button-matrix row watch mask, [2:0] are used */
  BYTE          bMatrixColMask;     /* Button-matrix column watch mask, [5:0] are used */
  tBMsr_PtsElement xdata *prPtsReserveHead; 	/* Allocated PTS array pointer */
  BYTE          bPtsSize;         	/* Number of entries to be in PTS */
  BYTE          bPushQualThresh;  	/* # consec button states to qualify push */
} tBMsr_Setup;


/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION PROTOTYPES:
 */

/* ---- Button Service Routine (BSR) */

WORD    wBMsr_Pop        /* Returns WORD: MSB timestamp, LSB button vector */
        (
        void
        );

void    vBMsr_Setup
        (
        tBMsr_Setup xdata *priSetup  /* Setup structure */
        );

void    vBMsr_InitPts    /* Initialize PTS */
        (
        BYTE    biPtsSize
        );

BYTE    bBMsr_GetPtsItemCnt   /* Returns BYTE: Number of unpopped entries in PTS */
        (
        void
        );

void    vBMsr_Service
        (
        void
        );

/* Returns WORD: MSB timestamp, LSB button vector of the latest button push */
WORD    wBMsr_GetCurrentButton
        (
        void
        );

BYTE    bBMsr_GetTimestamp
        (
        void
        );

/*
 *------------------------------------------------------------------------------
 */
#endif /* _SI4010_API_BMSR */


