/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                               Copyright 2010
 *                         CONFIDENTIAL & PROPRIETARY
 *------------------------------------------------------------------------------
 *
 *    FILE:       OD_main.h
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    RELEASE:    1.1 ; ROM version 02.00
 *    AUTHOR:     Miklos Lukacs
 *    DATE:       May 31, 2010
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      Header file for the Overlay demo main module.
 *
 *------------------------------------------------------------------------------*/


#ifndef _OVERLAY_DEMO_MAIN_H
#define _OVERLAY_DEMO_MAIN_H


/*------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include "si4010_types.h"

/*------------------------------------------------------------------------------
 *
 *    Definitions:
 */


#define bRke_EncodeNoneNrz_c       0   // No encoding //
#define bRke_EncodeManchester_c    1   // Manchester encoding //
#define bRke_Encode4b5b_c          2   // 4b-5b encoding //

#define bRke_OokMod_c        (0x00)
#define bRke_FskMod_c        (0x01)

// Amount of time (x32ms), the application will wait after boot without
// getting a qualified button push before giving up and shutting the chip down.
// Sets the maximum wait time for push as 1.6s; the variable is unit in 32ms.
#define bMaxWaitForPush         50

// This specifies the number of times that the RTC ISR is called between 
// each call to the button service routine.  The actual time between
// calls is dependent on the setting of RTC_CTRL.RTC_DIV which dictates how
// often the RTC ISR is called.    
#define bDebounceInterval 		8





/*------------------------------------------------------------------------------
 *
 *    VARIABLES:
 */

// Structure for setting up the ODS .. must be in XDATA //
  tOds_Setup xdata rOdsSetup;

// Structure for setting up the PA .. must be in XDATA //
  tPa_Setup xdata  rPaSetup;

// Battery measurement .. delete if battery measurement is not used //
  int           iBatteryMv;
  BYTE          bBatteryWait;   // Wait time after loading in 17us increment //

// BSR control structure //
  tBsr_Setup      xdata rBsrSetup;
  BYTE xdata abBsrPtsPlaceHolder [20] = {0};

// setup structure for external crystal oscilator //
  tFCast_XoSetup rFCast_XoSetup;

 BYTE bIsr_DebounceCount;
 BYTE bdata bPrevBtn;
 bit gAp_NewPushFlag;
 bit gAp_NonePushFlag;
 BYTE xdata bAp_PrevButtonState;
 BYTE xdata bAp_ButtonStateDiff;

/*-------------------------------------------------------------------------
 *
 *    FUNCTION PROTOTYPES:										
 */

void vAppInitial(void);

void vButtonCheck(void);



/*------------------------------------------------------------------------------
/*------------------------------------------------------------------------------
 *
 *    OVERLAY REGION
 *
 *------------------------------------------------------------------------------
 *-----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
 *
 *    Definitions:
 */

// Select how to call overlay function 

//#define AssemblyCall           
#define FuncPointerCall

// Define the NVM load addresses of the overlay functions
#define wNvmAddr_PacketAssembler   	0xE490
#define wNvmAddr_SendPacket        	0xE560

// Type declaration for a pointer points to a function/
typedef void (tOvrl_Func)(void);

/*------------------------------------------------------------------------------
 *
 *    VARIABLES:
 */

// Definition of the pointer to a function
tOvrl_Func *pOvrl_Func;           

// Declare a variable storing the NVM load address of overlay functions
WORD wNvmAddrOvrl;       

/*-------------------------------------------------------------------------
 *
 *    FUNCTION PROTOTYPES:										
 */

//  will be used as overlay
void vOvrl_FuncReserve(void);   

// Declare the function written in assembly, what will
// be used to call the function in the overlay module
void vOvrl_FuncCaller(void);

// Declare the function what will load the overlay from NVM
BYTE vOvrl_NvmLoad ( WORD  wiNvmAddr );


#endif // _OVERLAY_DEMO_MAIN_H
