/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       OD_ovrl1.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    RELEASE:    1.1 ; ROM version 02.00
 *    AUTHOR:     Miklos Lukacs
 *    DATE:       May 31, 2010
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the Overlay 1 Module of the Overlay demo.
 *      Handles the RF packet construction:
 *
 *      BL51 linker directives for building this application:
 *
 *            PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0500 - 0x0FFF)
 *            DATA (0x60)
 *            IDATA (0x80)
 *            XDATA (0x1000-0x106F)
 *            STACK (?STACK(0xA0)); dummy setting, THE STACK POINTER IS NOT SET!!!
 *                                  in overlay_ovrl1_startup.a51
 */


/*------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"

#include "OD_ovrl1.h"
#include "OD_global.h"



/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This is the main() function in the Overlay 1 Module of the Overlay Demo.
 *      It is copied into RAM and called after vButtonCheck() in the Main Module, to update  
 *      the packet for transmission, according to the button pressed.
 *		Realizes the void vPacketAssemble ( void ) function of the Keyfob Demo.
 *
 *------------------------------------------------------------------------------ 
 */
void    main( void)
{
  /* Normal packet transmission mode, send the associated packet
   * according to different button pressed.
   * For the demo application in conjunction with si4320 receiver,
   * there are five different packets corresponding to the five buttons.
   * GPIO0 .. Button1 .. AA AA AA 2D D4 D7 28 D7
   * GPIO1 .. Button2 .. AA AA AA 2D D4 F5 0A F5
   * GPIO2 .. Button3 .. AA AA AA 2D D4 5F A0 5F
   * GPIO3 .. Button4 .. AA AA AA 2D D4 7D 82 7D
   * GPIO4 .. Button5 .. AA AA AA 2D D4 FF 00 FF
   *
   * Note that Output Data Serializer (ODS) is in little endian fashion,
   * bit 0 first and inverted. So each byte of the packet data should be
   * converted properly before writing into frame buffer.
   */

    switch (bAp_ButtonState)
    {
    case bButton1_c:
      lPayLoad = lFramePayLoad1_c;
      break;
    case bButton2_c:
      lPayLoad = lFramePayLoad2_c;
      break;
    case bButton3_c:
      lPayLoad = lFramePayLoad3_c;
      break;
    case bButton4_c:
      lPayLoad = lFramePayLoad4_c;
      break;
    case bButton5_c:
      lPayLoad = lFramePayLoad5_c;
      break;
    default:
      lPayLoad = 0;
      break;
    }

    // Fill frame buffer with the corresponding button data.
    abButtonFrame[0] = (lFramePreamble_c >> 16) & 0xFF;
    abButtonFrame[1] = (lFramePreamble_c >> 8) & 0xFF;
    abButtonFrame[2] = lFramePreamble_c & 0xFF;
    abButtonFrame[3] = (wFrameSyncWord_c >> 8) & 0xFF;
    abButtonFrame[4] = wFrameSyncWord_c & 0xFF;
    abButtonFrame[5] = ((BYTE *) &lPayLoad)[1];
    abButtonFrame[6] = ((BYTE *) &lPayLoad)[2];
    abButtonFrame[7] = ((BYTE *) &lPayLoad)[3];

	return;
}
