/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       OD_ovrl2.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    RELEASE:    1.1 ; ROM version 02.00
 *    AUTHOR:     Miklos Lukacs
 *    DATE:       May 31, 2010
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the Overlay 2 Module of the Overlay demo.
 *         - Transmits the RF packet  
 *         - tunes to the center frequency,
 *         - sets and keeps the accuracy of the FSK deviation
 *         - tunes the PA
 *
 *      BL51 linker directives for building this application:
 *
 *            PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0500 - 0x0FFF)
 *            DATA (0x70)
 *            IDATA (0x80)
 *            XDATA (0x1000-0x106F)
 *            STACK (?STACK(0xA0)); dummy setting, THE STACK POINTER IS NOT SET!!!
 *                                  in overlay_ovrl1_startup.a51
 */


/*------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include "si4010.h"
#include "si4010_api_rom.h"

#include "OD_ovrl2.h"
#include "OD_global.h"


/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This is the main() function in the Overlay 2 Module of the Overlay Demo.
 *      It is copied into RAM and called after the RF packet construction in the 
 *      Overlay 1 Module by the Main Module.
 *      It realizes the vRtl_RepeatTxLoop() function of the Keyfob Demo, and loop which consists of three procedures,
 *      executes three task in a loop:
 *          - TX setup, 
 *          - TX one packet
 *          - Stop TX.
 *
 *------------------------------------------------------------------------------ 
 */
void    main( void )
{
  bRepeatCount = bRepeatCount_c;
  do
  {
    // Setup the bandgap  
    vSys_BandGapLdo( 1 );

    // Save current timestamp for repeat transmission interval checking later. 
    lTimestamp = lSys_GetMasterTime();

    // Run at 433.46 MHz (Upper frequency).Input is in [Hz]. 
	// The function also calculates necessary data to be used by the single Tx loop 
	// main function, vStl_SingleTxLoop(), for fine frequency correction during TX
    vFCast_Tune( fOperatingFrequency );

    // bFskDeviation comes from the calculation spreadsheet.
    vFCast_FskAdj( bFskDeviation ); 	

    // Now wait until there is a temperature value  
    while ( 0 == bDmdTs_GetSamplesTaken() ){}

    // Tune the PA with the current temperature as an argument 
    vPa_Tune( iDmdTs_GetLatestTemp());

    // Sets up the temperature sensor and demodulator,
    // enable ODS before entering the single tx loop. 
    vStl_PreLoop();

    // Then assign the first address of the frame buffer to
    // the packet pointer used for transmission. 
    pbRke_FrameHead = abButtonFrame;

    // call the single tx loop function 
    vStl_SingleTxLoop(pbRke_FrameHead,
                      bFrameSize_c);

    // Disable the temperature sensor and ODS
    vStl_PostLoop();

    // Disable Bandgap and LDO 
     vSys_BandGapLdo(0);

     //turn LED on
	 GPIO_LED = 1; 
	       
    // Wait repeat interval. 
    while ( (lSys_GetMasterTime() - lTimestamp) < wRepeatInterval );

  }while(--bRepeatCount);

  // After repeat transmission, clear all tasks flag. 
  bAp_ButtonState = 0;

  // Clear time value for next button push detecting. 
  vSys_SetMasterTime(0);

  //turn LED off
  GPIO_LED = 0; 
    
  return;
} 

