;------------------------------------------------------------------------------
;                          Silicon Laboratories, Inc.
;                           http://www.silabs.com
;                               Copyright 2010
;------------------------------------------------------------------------------
;
;     FILE:       ovrl_caller.asm
;     TARGET:     Si4010
;     TOOLCHAIN:  Keil
;     RELEASE:    1.0
;     AUTHOR:     Attila Gosi
;     DATE:       May 19, 2010, Tuesday
;
;------------------------------------------------------------------------------
;
;     DESCRIPTION:
;       This file saves the relevant MCU registers to Stack, calls the overlay
;       function, and restores the registers. It ensures that the MCU
;       will be in the same state before and after the Main -> Overlay -> 
;       Main context switching sequence. 
;
;------------------------------------------------------------------------------




;
;------------------------------------------------------------------------------
;
EXTRN	CODE(vOvrl_FuncReserve)
PUBLIC	vOvrl_FuncCaller
;
;------------------------------------------------------------------------------
;

;
;------------------------------------------------------------------------------
;
?PR?VOVRL_FUNCCALLER?OVRL_CALLER		SEGMENT CODE 
	RSEG  				?PR?VOVRL_FUNCCALLER?OVRL_CALLER
	USING	0

;-------------------------------------------------------------------------------
; 
;   Store accumulators (Acc, B), data pointer (DPH, DPL), program status word 
;   (PSW), and general purpose registers (R0-R7) in stack, call overlay
;   function, restore registers from stack. Storing and restoring these
;   registers is NOT mandatory.
;
;-------------------------------------------------------------------------------

vOvrl_FuncCaller:

	PUSH 	ACC			; store the context
	PUSH 	B
	PUSH 	DPH
	PUSH 	DPL
	PUSH 	PSW
	MOV  	PSW,#00H
	PUSH 	AR0
	PUSH 	AR1
	PUSH 	AR2
	PUSH 	AR3
	PUSH 	AR4
	PUSH 	AR5
	PUSH 	AR6
	PUSH 	AR7
	USING	0

  LCALL vOvrl_FuncReserve     ; call overlay function

	POP  	AR7		   ; restore the context
	POP  	AR6
	POP  	AR5
	POP  	AR4
	POP  	AR3
	POP  	AR2
	POP  	AR1
	POP  	AR0
	POP  	PSW
	POP  	DPL
	POP  	DPH
	POP  	B
	POP  	ACC

	RET
	
	END

;
;------------------------------------------------------------------------------
;




