@REM ---------------------------------------------------------------------------
@REM                        Silicon Laboratories, Inc.
@REM                             Copyright 2010
@REM                       CONFIDENTIAL & PROPRIETARY
@REM ---------------------------------------------------------------------------
@REM
@REM ---------------------------------------------------------------------------
@REM
@REM    DESCRIPTION:
@REM      Batch file that accepts a name of the configuration
@REM      C file without extension to build. The C file must reside in ../src/
@REM      directory. For example:
@REM        build_config  config_data
@REM      creates
@REM        ..\out\config_data.hex
@REM      configuration file from the input ..\src\config_data.c
@REM
@REM ---------------------------------------------------------------------------

@ECHO OFF
ECHO.---------------------------------------------------------------------------
ECHO.  (c) 2010 Silicon Laboratories, Inc.
ECHO.         - configuration build -
ECHO.---------------------------------------------------------------------------
ECHO.  Timestamp: %DATE% %TIME%
ECHO.

REM -- Source and output directories
SET _SrcDir_=..\src
SET _OutDir_=..\out

REM -- Settings of the tools and options
SET _KEILPATH_=C:\Keil\C51\BIN
SET _C51OPTIONS_= BROWSE NOINTPROMOTE NOAREGS INCDIR(%_SrcDir_%) DEBUG OBJECTEXTEND CODE PRINT(%_OutDir_%\%1.lst) TABS(8) OBJECT(%_OutDir_%\%1.obj)
SET _BL51OPTIONS_=PRINT (%_OutDir_%\%1.m51) RAMSIZE (256) NOOVERLAY CODE (?CO?%1 (0X0500))

REM -- Check the input
SET _OutputObjFile_=%_OutDir_%\%1.obj
SET _OutputOmfFile_=%_OutDir_%\%1.omf
SET _OutputHexFile_=%_OutDir_%\%1.hex
SET _InputFile_=%_SrcDir_%\%1.c

REM Check input file existence
IF [%1]==[] GOTO PRINT_HELP
IF [%1]==[/?] GOTO PRINT_HELP
IF NOT EXIST %_InputFile_% GOTO FILE_ERROR

REM -- Build
ECHO.  -- Building: (%_InputFile_%) --^> (%_OutputHexFile_%)
ECHO.

REM -- Compile, link, and IntelHEX conversion
ECHO.  -- Compiling: (%_InputFile_%)

%_KEILPATH_%\C51.exe %_InputFile_% %_C51OPTIONS_%

ECHO.
ECHO.  -- Linking: (%_OutputOmfFile_%)

%_KEILPATH_%\BL51.exe %_OutputObjFile_% TO %_OutputOmfFile_% %_BL51OPTIONS_%

ECHO.
ECHO.  -- HEX conversion: (%_OutputHexFile_%)

%_KEILPATH_%\OH51.exe %_OutputOmfFile_% HEXFILE (%_OutputHexFile_%)

ECHO.
ECHO.  -- Removing OMF and OBJ files

DEL %_OutputOmfFile_%
DEL %_OutputObjFile_%

GOTO QUITBUILD

REM --------

:PRINT_HELP
ECHO.
ECHO.  Usage: build_config config_name
ECHO.    config_name .. string denoting file (..\src\config_name.c)
ECHO.
GOTO QUITBUILD

:FILE_ERROR
ECHO.
ECHO.  ERROR: Input file (%_InputFile_%) does not exist. Nothing build.
ECHO.
GOTO QUITBUILD

:QUITBUILD
ECHO.
ECHO.  -- Done

@REM
@REM ---------------------------------------------------------------------------
@REM
